import ddf.minim.*;
import ddf.minim.signals.*;
import ddf.minim.analysis.*;
import ddf.minim.effects.*;

Player boli;
Enemy bolinhas[];
Quadrado quadradinho;
int r, g, b;
int loadPercentage;
boolean loading;

Minim minim;
AudioPlayer player;

void setup() {
    size(640, 480);
    background(0, 0, 0);
    smooth();
    
    loading = true;
    loadPercentage = 0;
    //Background
    r = 0;
    g = 0;
    b = 0;
    
    boli = new Player(width/2, height/2, 30);
    bolinhas = new Enemy[50];
    for(int i = 0; i < 50; i++) {
      bolinhas[i] = new Enemy((int)random(30));
    }
    
    quadradinho = new Quadrado((int)random(50));
    
    minim = new Minim(this);
    player = minim.loadFile("xibata.mp3");
    player.play();
    player.loop();
}

void draw() {
  
  
  if (loading) {
    background(0, 0, 0);
    PFont font = loadFont("ArialMT-48.vlw"); 
    textFont(font, 32); 
    String loadingText = "...loading, please wait...";
    text(loadingText, width/2 - 150, height/2);
    text(str(loadPercentage) + "%", width/2-100, height/2 + 50);
    if (random(100) < 2) {
      loadPercentage++;
    }
    
    if (loadPercentage >= 100) {
      loading = false;
      player.close();
      player = minim.loadFile("EpicArpg.mp3");
      player.play();
      player.loop();
    }
    
  } else {
    background(r, g, b);
  
  //Criar inimigos
  for (int i = 0; i < 50; i++) {
    if(random(100) < 20) {
      bolinhas[i].activate();
    }
    bolinhas[i].update();
  }
  
  //Criar quadrados, percentagem 5%
  if (!quadradinho.activated && (random(100) < 5)) {
    quadradinho = new Quadrado((int)random(10));
    quadradinho.activate();
  }
  quadradinho.update();  
  boli.update();
  
  if (r > 0) {
    r -= 1;
  }
  if (g > 0) {
    g -= 1;
  } 
  if (b > 0) {
    b -= 1;
  }
  }
  
}

void keyPressed() {
  if (key == CODED) {
    if (keyCode == UP) {
      boli.moveUp();
    } 
    
    if (keyCode == DOWN) {
      boli.moveDown();
    } 
    
    if (keyCode == LEFT) {
      boli.moveLeft();
    } 
    
    if (keyCode == RIGHT) {
      boli.moveRight();
    } 
  } else{
    boli.decreaseAccel();
  }
}


void desenharBolinha() {
  fill(random(255), random(255), random(255));
  ellipse(boli.x, boli.y, 80, 80);
}

class Enemy {
  int x,y;
  int speed;
  boolean activated;
  
  public Enemy(int fallingSpeed) {
    x = (int)random(width);
    y = 0;
    speed = fallingSpeed;
    activated = false;
  }
  
  void update() {
    
    if(activated) {
      y += speed;
      fill(random(255));
      ellipse(x, y, 20, 20);
    }
    
    if (activated && (y > height)) {
      deactivate();
    }
    
    //ColisÃ£o
    if (boli.checkCollision(x, y)) {
      //Desenhar linhas aleatorias
      for(int i = 0; i < 30; i++) {
        stroke(random(255), random(255), random(255));
        line(x, y, random(width), random(height));
      }
    }
    
  }
  
  void activate() {
    
    if (!activated) {
      x = (int)random(width);
      activated = true;
    }
  }
  
  void deactivate() {
    activated = false;
    y = 0;
  }
}

class Player {
  
  int x, y;
  int speed;
  int accel;
  int max_accel;
  
  public Player(int start_x, int start_y, int max_accel) {
    x = start_x;
    y = start_y;
    accel = 0;
    this.max_accel = max_accel;
  }
  
  void moveUp() {
    increaseAccel();    
    y -= accel;
  }
  
  void moveDown() {
    increaseAccel();    
    y += accel;
  }
  
  void moveLeft() {
    increaseAccel();    
    x -= accel;
  }
  
  void moveRight() {
    increaseAccel();    
    x += accel;
  }
  
  
  void decreaseAccel() {
    if (accel > 0) {
      accel -= 1;
    }
  }
  
  void increaseAccel() {
    if (accel < max_accel) {
      accel += 1;
    } 
  }
  
  void update(){
    if (x > width) {
      x = 0;
    } else if (x < 0) {
      x = width;
    }
    
    if (y > height) {
      y = 0;
    } else if (y < 0) {
      y = height;
    }
    
    desenharBolinha();
  }
  
  boolean checkCollision(int x, int y) {
    return ((x >= boli.x - 40) && (x <= boli.x + 40)) && ((y >= boli.y - 40) && (y <= boli.y + 40));
  }
}

class GeneralGeometry {
  int x, y, speed;
  
  public GeneralGeometry(int x, int y, int speed) {
    this.x = x;
    this.y = y;
    this.speed = speed;
  }
  
  void moveDown() {
    y += speed;
  } 
  
  void moveUp() {
    y -= speed;
  }
  
  void moveLeft() {
    x -= speed;
  }
  
  void moveRight() {
    x += speed;
  }
  
}

class Quadrado extends GeneralGeometry {
  
  boolean activated;
  
  public Quadrado(int speed) {
    super((int)random(width), 0, speed);
    activated = false;
  }
  
  void update() {
    //Verificar colisÃ£o
    if (boli.checkCollision(super.x, super.y)) {
      r = (int)random(255);
      g = (int)random(255);
      b = (int)random(255);
      deactivate();
    }
    
    if(activated) {
      super.moveDown();
    }
    
    if (y > height) {
      deactivate();
    }
    
    //Desenhar
    fill(random(255), random(255), random(255));
    rect(super.x, super.y, 30, 30);
    
  }
  
  void activate() {
    activated = true;
  } 
  
  void deactivate() {
    activated = false;
    super.y = 0 - 30;
    super.x = (int)random(width);
  }
}

void stop(){
  player.close();
  minim.stop();
  super.stop();
}
